#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Mikhail Yohman (@fragmentedpacket) <mikhail.yohman@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: netbox_ip_address
short_description: Creates or removes IP addresses from Netbox
description:
  - Creates or removes IP addresses from Netbox
notes:
  - Tags should be defined as a YAML list
  - This should be ran with connection C(local) and hosts C(localhost)
author:
  - Mikhail Yohman (@FragmentedPacket)
  - Anthony Ruhier (@Anthony25)
requirements:
  - pynetbox
version_added: '2.8'
options:
  netbox_url:
    description:
      - URL of the Netbox instance resolvable by Ansible control host
    required: true
  netbox_token:
    description:
      - The token created within Netbox to authorize API access
    required: true
  data:
    description:
      - Defines the IP address configuration
    suboptions:
      family:
        description:
          - Specifies with address family the IP address belongs to
        choices:
          - 4
          - 6
      address:
        description:
          - Required if state is C(present)
      prefix:
        description:
          - |
            With state C(present), if an interface is given, it will ensure
            that an IP inside this prefix (and vrf, if given) is attached
            to this interface. Otherwise, it will get the next available IP
            of this prefix and attach it.
            With state C(new), it will force to get the next available IP in
            this prefix. If an interface is given, it will also force to attach
            it.
            Required if state is C(present) or C(new) when no address is given.
            Unused if an address is specified.
      vrf:
        description:
          - VRF that IP address is associated with
      tenant:
        description:
          - The tenant that the device will be assigned to
      status:
        description:
          - The status of the IP address
        choices:
          - Active
          - Reserved
          - Deprecated
          - DHCP
      role:
        description:
          - The role of the IP address
        choices:
          - Loopback
          - Secondary
          - Anycast
          - VIP
          - VRRP
          - HSRP
          - GLBP
          - CARP
      interface:
        description:
          - |
            The name and device of the interface that the IP address should be assigned to
            Required if state is C(present) and a prefix specified.
      description:
        description:
          - The description of the interface
      nat_inside:
        description:
          - The inside IP address this IP is assigned to
      tags:
        description:
          - Any tags that the IP address may need to be associated with
      custom_fields:
        description:
          - must exist in Netbox
    required: true
  state:
    description:
      - |
        Use C(present), C(new) or C(absent) for adding, force adding or removing.
        C(present) will check if the IP is already created, and return it if
        true. C(new) will force to create it anyway (useful for anycasts, for
        example).
    choices: [ absent, new, present ]
    default: present
  validate_certs:
    description:
      - If C(no), SSL certificates will not be validated. This should only be used on personally controlled sites using self-signed certificates.
    default: 'yes'
    type: bool
'''

EXAMPLES = r'''
- name: "Test Netbox IP address module"
  connection: local
  hosts: localhost
  gather_facts: False

  tasks:
    - name: Create IP address within Netbox with only required information
      netbox_ip_address:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          address: 192.168.1.10
        state: present
    - name: Force to create (even if it already exists) the IP
      netbox_ip_address:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          address: 192.168.1.10
        state: new
    - name: Get a new available IP inside 192.168.1.0/24
      netbox_ip_address:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          prefix: 192.168.1.0/24
        state: new
    - name: Delete IP address within netbox
      netbox_ip_address:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          address: 192.168.1.10
        state: absent
    - name: Create IP address with several specified options
      netbox_ip_address:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          family: 4
          address: 192.168.1.20
          vrf: Test
          tenant: Test Tenant
          status: Reserved
          role: Loopback
          description: Test description
          tags:
            - Schnozzberry
        state: present
    - name: Create IP address and assign a nat_inside IP
      netbox_ip_address:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          family: 4
          address: 192.168.1.30
          vrf: Test
          nat_inside:
            address: 192.168.1.20
            vrf: Test
          interface:
            name: GigabitEthernet1
            device: test100
    - name: Ensure that an IP inside 192.168.1.0/24 is attached to GigabitEthernet1
      netbox_ip_address:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          prefix: 192.168.1.0/24
          vrf: Test
          interface:
            name: GigabitEthernet1
            device: test100
        state: present
    - name: Attach a new available IP of 192.168.1.0/24 to GigabitEthernet1
      netbox_ip_address:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          prefix: 192.168.1.0/24
          vrf: Test
          interface:
            name: GigabitEthernet1
            device: test100
        state: new
'''

RETURN = r'''
ip_address:
  description: Serialized object as created or already existent within Netbox
  returned: on creation
  type: dict
msg:
  description: Message indicating failure or info about what has been achieved
  returned: always
  type: str
'''

import json
import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.net_tools.netbox.netbox_utils import (
    find_ids,
    normalize_data,
    create_netbox_object,
    delete_netbox_object,
    update_netbox_object,
    IP_ADDRESS_ROLE,
    IP_ADDRESS_STATUS
)
from ansible.module_utils.compat import ipaddress
from ansible.module_utils._text import to_text


PYNETBOX_IMP_ERR = None
try:
    import pynetbox
    HAS_PYNETBOX = True
except ImportError:
    PYNETBOX_IMP_ERR = traceback.format_exc()
    HAS_PYNETBOX = False


def main():
    '''
    Main entry point for module execution
    '''
    argument_spec = dict(
        netbox_url=dict(type="str", required=True),
        netbox_token=dict(type="str", required=True, no_log=True),
        data=dict(type="dict", required=True),
        state=dict(required=False, default='present', choices=['present', 'absent', 'new']),
        validate_certs=dict(type="bool", default=True)
    )

    global module
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    # Fail module if pynetbox is not installed
    if not HAS_PYNETBOX:
        module.fail_json(msg=missing_required_lib('pynetbox'), exception=PYNETBOX_IMP_ERR)

    # Assign variables to be used with module
    changed = False
    app = 'ipam'
    endpoint = 'ip_addresses'
    url = module.params["netbox_url"]
    token = module.params["netbox_token"]
    data = module.params["data"]
    state = module.params["state"]
    validate_certs = module.params["validate_certs"]

    # Attempt to create Netbox API object
    try:
        nb = pynetbox.api(url, token=token, ssl_verify=validate_certs)
    except Exception:
        module.fail_json(msg="Failed to establish connection to Netbox API")
    try:
        nb_app = getattr(nb, app)
    except AttributeError:
        module.fail_json(msg="Incorrect application specified: %s" % (app))

    nb_endpoint = getattr(nb_app, endpoint)
    norm_data = normalize_data(data)
    try:
        norm_data = _check_and_adapt_data(nb, norm_data)
        if state in ("new", "present"):
            return _handle_state_new_present(
                module, state, nb_app, nb_endpoint, norm_data
            )
        elif state == "absent":
            return module.exit_json(
                **ensure_ip_address_absent(nb_endpoint, norm_data)
            )
        else:
            return module.fail_json(msg="Invalid state %s" % state)
    except pynetbox.RequestError as e:
        return module.fail_json(msg=json.loads(e.error))
    except ValueError as e:
        return module.fail_json(msg=str(e))


def _check_and_adapt_data(nb, data):
    data = find_ids(nb, data)

    if data.get("vrf") and not isinstance(data["vrf"], int):
        raise ValueError(
            "%s does not exist - Please create VRF" % (data["vrf"])
        )
    if data.get("status"):
        data["status"] = IP_ADDRESS_STATUS.get(data["status"].lower())
    if data.get("role"):
        data["role"] = IP_ADDRESS_ROLE.get(data["role"].lower())

    return data


def _handle_state_new_present(module, state, nb_app, nb_endpoint, data):
    if data.get("address"):
        if state == "present":
            return module.exit_json(
                **ensure_ip_address_present(nb_endpoint, data)
            )
        elif state == "new":
            return module.exit_json(
                **create_ip_address(nb_endpoint, data)
            )
    else:
        if state == "present":
            return module.exit_json(
                **ensure_ip_in_prefix_present_on_netif(
                    nb_app, nb_endpoint, data
                )
            )
        elif state == "new":
            return module.exit_json(
                **get_new_available_ip_address(nb_app, data)
            )


def ensure_ip_address_present(nb_endpoint, data):
    """
    :returns dict(ip_address, msg, changed): dictionary resulting of the request,
    where 'ip_address' is the serialized ip fetched or newly created in Netbox
    """
    if not isinstance(data, dict):
        changed = False
        return {"msg": data, "changed": changed}

    try:
        nb_addr = _search_ip(nb_endpoint, data)
    except ValueError:
        return _error_multiple_ip_results(data)

    result = {}
    if not nb_addr:
        return create_ip_address(nb_endpoint, data)
    else:
        ip_addr, diff = update_netbox_object(nb_addr, data, module.check_mode)
        if ip_addr is False:
            module.fail_json(
                msg="Request failed, couldn't update IP: %s" % (data["address"])
            )
        if diff:
            msg = "IP Address %s updated" % (data["address"])
            changed = True
            result["diff"] = diff
        else:
            ip_addr = nb_addr.serialize()
            changed = False
            msg = "IP Address %s already exists" % (data["address"])

        return {"ip_address": ip_addr, "msg": msg, "changed": changed}


def _search_ip(nb_endpoint, data):
    get_query_params = {"address": data["address"]}
    if data.get("vrf"):
        get_query_params["vrf_id"] = data["vrf"]

    ip_addr = nb_endpoint.get(**get_query_params)
    return ip_addr


def _error_multiple_ip_results(data):
    changed = False
    if "vrf" in data:
        return {"msg": "Returned more than result", "changed": changed}
    else:
        return {
            "msg": "Returned more than one result - Try specifying VRF.",
            "changed": changed
        }


def create_ip_address(nb_endpoint, data):
    if not isinstance(data, dict):
        changed = False
        return {"msg": data, "changed": changed}

    ip_addr, diff = create_netbox_object(nb_endpoint, data, module.check_mode)
    changed = True
    msg = "IP Addresses %s created" % (data["address"])

    return {"ip_address": ip_addr, "msg": msg, "changed": changed, "diff": diff}


def ensure_ip_in_prefix_present_on_netif(nb_app, nb_endpoint, data):
    """
    :returns dict(ip_address, msg, changed): dictionary resulting of the request,
    where 'ip_address' is the serialized ip fetched or newly created in Netbox
    """
    if not isinstance(data, dict):
        changed = False
        return {"msg": data, "changed": changed}

    if not data.get("interface") or not data.get("prefix"):
        raise ValueError("A prefix and interface are required")

    get_query_params = {
        "interface_id": data["interface"], "parent": data["prefix"],
    }
    if data.get("vrf"):
        get_query_params["vrf_id"] = data["vrf"]

    attached_ips = nb_endpoint.filter(**get_query_params)
    if attached_ips:
        ip_addr = attached_ips[-1].serialize()
        changed = False
        msg = "IP Address %s already attached" % (ip_addr["address"])

        return {"ip_address": ip_addr, "msg": msg, "changed": changed}
    else:
        return get_new_available_ip_address(nb_app, data)


def get_new_available_ip_address(nb_app, data):
    prefix_query = {"prefix": data["prefix"]}
    if data.get("vrf"):
        prefix_query["vrf_id"] = data["vrf"]

    result = {}
    prefix = nb_app.prefixes.get(**prefix_query)
    if not prefix:
        changed = False
        msg = "%s does not exist - please create first" % (data["prefix"])
        return {"msg": msg, "changed": changed}
    elif prefix.available_ips.list():
        ip_addr, diff = create_netbox_object(prefix.available_ips, data, module.check_mode)
        changed = True
        msg = "IP Addresses %s created" % (ip_addr["address"])
        result["diff"] = diff
    else:
        changed = False
        msg = "No available IPs available within %s" % (data['prefix'])
        return {"msg": msg, "changed": changed}

    result.update({"ip_address": ip_addr, "msg": msg, "changed": changed})
    return result


def _get_prefix_id(nb_app, prefix, vrf_id=None):
    ipaddr_prefix = ipaddress.ip_network(prefix)
    network = to_text(ipaddr_prefix.network_address)
    mask = ipaddr_prefix.prefixlen

    prefix_query_params = {
        "prefix": network,
        "mask_length": mask
    }
    if vrf_id:
        prefix_query_params["vrf_id"] = vrf_id

    prefix_id = nb_app.prefixes.get(prefix_query_params)
    if not prefix_id:
        if vrf_id:
            raise ValueError("Prefix %s does not exist in VRF %s - Please create it" % (prefix, vrf_id))
        else:
            raise ValueError("Prefix %s does not exist - Please create it" % (prefix))

    return prefix_id


def ensure_ip_address_absent(nb_endpoint, data):
    """
    :returns dict(msg, changed)
    """
    if not isinstance(data, dict):
        changed = False
        return {"msg": data, "changed": changed}

    try:
        ip_addr = _search_ip(nb_endpoint, data)
    except ValueError:
        return _error_multiple_ip_results(data)

    result = {}
    if ip_addr:
        dummy, diff = delete_netbox_object(ip_addr, module.check_mode)
        changed = True
        msg = "IP Address %s deleted" % (data["address"])
        result["diff"] = diff
    else:
        changed = False
        msg = "IP Address %s already absent" % (data["address"])

    result.update({"msg": msg, "changed": changed})
    return result


if __name__ == "__main__":
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_voip_profile
short_description: Configure VoIP profiles in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify voip feature and profile category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    voip_profile:
        description:
            - Configure VoIP profiles.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            comment:
                description:
                    - Comment.
            name:
                description:
                    - Profile name.
                required: true
            sccp:
                description:
                    - SCCP.
                suboptions:
                    block-mcast:
                        description:
                            - Enable/disable block multicast RTP connections.
                        choices:
                            - disable
                            - enable
                    log-call-summary:
                        description:
                            - Enable/disable log summary of SCCP calls.
                        choices:
                            - disable
                            - enable
                    log-violations:
                        description:
                            - Enable/disable logging of SCCP violations.
                        choices:
                            - disable
                            - enable
                    max-calls:
                        description:
                            - Maximum calls per minute per SCCP client (max 65535).
                    status:
                        description:
                            - Enable/disable SCCP.
                        choices:
                            - disable
                            - enable
                    verify-header:
                        description:
                            - Enable/disable verify SCCP header content.
                        choices:
                            - disable
                            - enable
            sip:
                description:
                    - SIP.
                suboptions:
                    ack-rate:
                        description:
                            - ACK request rate limit (per second, per policy).
                    block-ack:
                        description:
                            - Enable/disable block ACK requests.
                        choices:
                            - disable
                            - enable
                    block-bye:
                        description:
                            - Enable/disable block BYE requests.
                        choices:
                            - disable
                            - enable
                    block-cancel:
                        description:
                            - Enable/disable block CANCEL requests.
                        choices:
                            - disable
                            - enable
                    block-geo-red-options:
                        description:
                            - Enable/disable block OPTIONS requests, but OPTIONS requests still notify for redundancy.
                        choices:
                            - disable
                            - enable
                    block-info:
                        description:
                            - Enable/disable block INFO requests.
                        choices:
                            - disable
                            - enable
                    block-invite:
                        description:
                            - Enable/disable block INVITE requests.
                        choices:
                            - disable
                            - enable
                    block-long-lines:
                        description:
                            - Enable/disable block requests with headers exceeding max-line-length.
                        choices:
                            - disable
                            - enable
                    block-message:
                        description:
                            - Enable/disable block MESSAGE requests.
                        choices:
                            - disable
                            - enable
                    block-notify:
                        description:
                            - Enable/disable block NOTIFY requests.
                        choices:
                            - disable
                            - enable
                    block-options:
                        description:
                            - Enable/disable block OPTIONS requests and no OPTIONS as notifying message for redundancy either.
                        choices:
                            - disable
                            - enable
                    block-prack:
                        description:
                            - Enable/disable block prack requests.
                        choices:
                            - disable
                            - enable
                    block-publish:
                        description:
                            - Enable/disable block PUBLISH requests.
                        choices:
                            - disable
                            - enable
                    block-refer:
                        description:
                            - Enable/disable block REFER requests.
                        choices:
                            - disable
                            - enable
                    block-register:
                        description:
                            - Enable/disable block REGISTER requests.
                        choices:
                            - disable
                            - enable
                    block-subscribe:
                        description:
                            - Enable/disable block SUBSCRIBE requests.
                        choices:
                            - disable
                            - enable
                    block-unknown:
                        description:
                            - Block unrecognized SIP requests (enabled by default).
                        choices:
                            - disable
                            - enable
                    block-update:
                        description:
                            - Enable/disable block UPDATE requests.
                        choices:
                            - disable
                            - enable
                    bye-rate:
                        description:
                            - BYE request rate limit (per second, per policy).
                    call-keepalive:
                        description:
                            - Continue tracking calls with no RTP for this many minutes.
                    cancel-rate:
                        description:
                            - CANCEL request rate limit (per second, per policy).
                    contact-fixup:
                        description:
                            - "Fixup contact anyway even if contact's IP:port doesn't match session's IP:port."
                        choices:
                            - disable
                            - enable
                    hnt-restrict-source-ip:
                        description:
                            - Enable/disable restrict RTP source IP to be the same as SIP source IP when HNT is enabled.
                        choices:
                            - disable
                            - enable
                    hosted-nat-traversal:
                        description:
                            - Hosted NAT Traversal (HNT).
                        choices:
                            - disable
                            - enable
                    info-rate:
                        description:
                            - INFO request rate limit (per second, per policy).
                    invite-rate:
                        description:
                            - INVITE request rate limit (per second, per policy).
                    ips-rtp:
                        description:
                            - Enable/disable allow IPS on RTP.
                        choices:
                            - disable
                            - enable
                    log-call-summary:
                        description:
                            - Enable/disable logging of SIP call summary.
                        choices:
                            - disable
                            - enable
                    log-violations:
                        description:
                            - Enable/disable logging of SIP violations.
                        choices:
                            - disable
                            - enable
                    malformed-header-allow:
                        description:
                            - Action for malformed Allow header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-call-id:
                        description:
                            - Action for malformed Call-ID header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-contact:
                        description:
                            - Action for malformed Contact header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-content-length:
                        description:
                            - Action for malformed Content-Length header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-content-type:
                        description:
                            - Action for malformed Content-Type header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-cseq:
                        description:
                            - Action for malformed CSeq header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-expires:
                        description:
                            - Action for malformed Expires header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-from:
                        description:
                            - Action for malformed From header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-max-forwards:
                        description:
                            - Action for malformed Max-Forwards header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-p-asserted-identity:
                        description:
                            - Action for malformed P-Asserted-Identity header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-rack:
                        description:
                            - Action for malformed RAck header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-record-route:
                        description:
                            - Action for malformed Record-Route header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-route:
                        description:
                            - Action for malformed Route header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-rseq:
                        description:
                            - Action for malformed RSeq header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-a:
                        description:
                            - Action for malformed SDP a line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-b:
                        description:
                            - Action for malformed SDP b line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-c:
                        description:
                            - Action for malformed SDP c line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-i:
                        description:
                            - Action for malformed SDP i line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-k:
                        description:
                            - Action for malformed SDP k line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-m:
                        description:
                            - Action for malformed SDP m line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-o:
                        description:
                            - Action for malformed SDP o line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-r:
                        description:
                            - Action for malformed SDP r line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-s:
                        description:
                            - Action for malformed SDP s line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-t:
                        description:
                            - Action for malformed SDP t line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-v:
                        description:
                            - Action for malformed SDP v line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-sdp-z:
                        description:
                            - Action for malformed SDP z line.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-to:
                        description:
                            - Action for malformed To header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-header-via:
                        description:
                            - Action for malformed VIA header.
                        choices:
                            - discard
                            - pass
                            - respond
                    malformed-request-line:
                        description:
                            - Action for malformed request line.
                        choices:
                            - discard
                            - pass
                            - respond
                    max-body-length:
                        description:
                            - Maximum SIP message body length (0 meaning no limit).
                    max-dialogs:
                        description:
                            - Maximum number of concurrent calls/dialogs (per policy).
                    max-idle-dialogs:
                        description:
                            - Maximum number established but idle dialogs to retain (per policy).
                    max-line-length:
                        description:
                            - Maximum SIP header line length (78-4096).
                    message-rate:
                        description:
                            - MESSAGE request rate limit (per second, per policy).
                    nat-trace:
                        description:
                            - Enable/disable preservation of original IP in SDP i line.
                        choices:
                            - disable
                            - enable
                    no-sdp-fixup:
                        description:
                            - Enable/disable no SDP fix-up.
                        choices:
                            - disable
                            - enable
                    notify-rate:
                        description:
                            - NOTIFY request rate limit (per second, per policy).
                    open-contact-pinhole:
                        description:
                            - Enable/disable open pinhole for non-REGISTER Contact port.
                        choices:
                            - disable
                            - enable
                    open-record-route-pinhole:
                        description:
                            - Enable/disable open pinhole for Record-Route port.
                        choices:
                            - disable
                            - enable
                    open-register-pinhole:
                        description:
                            - Enable/disable open pinhole for REGISTER Contact port.
                        choices:
                            - disable
                            - enable
                    open-via-pinhole:
                        description:
                            - Enable/disable open pinhole for Via port.
                        choices:
                            - disable
                            - enable
                    options-rate:
                        description:
                            - OPTIONS request rate limit (per second, per policy).
                    prack-rate:
                        description:
                            - PRACK request rate limit (per second, per policy).
                    preserve-override:
                        description:
                            - "Override i line to preserve original IPS (default: append)."
                        choices:
                            - disable
                            - enable
                    provisional-invite-expiry-time:
                        description:
                            - Expiry time for provisional INVITE (10 - 3600 sec).
                    publish-rate:
                        description:
                            - PUBLISH request rate limit (per second, per policy).
                    refer-rate:
                        description:
                            - REFER request rate limit (per second, per policy).
                    register-contact-trace:
                        description:
                            - Enable/disable trace original IP/port within the contact header of REGISTER requests.
                        choices:
                            - disable
                            - enable
                    register-rate:
                        description:
                            - REGISTER request rate limit (per second, per policy).
                    rfc2543-branch:
                        description:
                            - Enable/disable support via branch compliant with RFC 2543.
                        choices:
                            - disable
                            - enable
                    rtp:
                        description:
                            - Enable/disable create pinholes for RTP traffic to traverse firewall.
                        choices:
                            - disable
                            - enable
                    ssl-algorithm:
                        description:
                            - Relative strength of encryption algorithms accepted in negotiation.
                        choices:
                            - high
                            - medium
                            - low
                    ssl-auth-client:
                        description:
                            - Require a client certificate and authenticate it with the peer/peergrp. Source user.peer.name user.peergrp.name.
                    ssl-auth-server:
                        description:
                            - Authenticate the server's certificate with the peer/peergrp. Source user.peer.name user.peergrp.name.
                    ssl-client-certificate:
                        description:
                            - Name of Certificate to offer to server if requested. Source vpn.certificate.local.name.
                    ssl-client-renegotiation:
                        description:
                            - Allow/block client renegotiation by server.
                        choices:
                            - allow
                            - deny
                            - secure
                    ssl-max-version:
                        description:
                            - Highest SSL/TLS version to negotiate.
                        choices:
                            - ssl-3.0
                            - tls-1.0
                            - tls-1.1
                            - tls-1.2
                    ssl-min-version:
                        description:
                            - Lowest SSL/TLS version to negotiate.
                        choices:
                            - ssl-3.0
                            - tls-1.0
                            - tls-1.1
                            - tls-1.2
                    ssl-mode:
                        description:
                            - SSL/TLS mode for encryption & decryption of traffic.
                        choices:
                            - off
                            - full
                    ssl-pfs:
                        description:
                            - SSL Perfect Forward Secrecy.
                        choices:
                            - require
                            - deny
                            - allow
                    ssl-send-empty-frags:
                        description:
                            - Send empty fragments to avoid attack on CBC IV (SSL 3.0 & TLS 1.0 only).
                        choices:
                            - enable
                            - disable
                    ssl-server-certificate:
                        description:
                            - Name of Certificate return to the client in every SSL connection. Source vpn.certificate.local.name.
                    status:
                        description:
                            - Enable/disable SIP.
                        choices:
                            - disable
                            - enable
                    strict-register:
                        description:
                            - Enable/disable only allow the registrar to connect.
                        choices:
                            - disable
                            - enable
                    subscribe-rate:
                        description:
                            - SUBSCRIBE request rate limit (per second, per policy).
                    unknown-header:
                        description:
                            - Action for unknown SIP header.
                        choices:
                            - discard
                            - pass
                            - respond
                    update-rate:
                        description:
                            - UPDATE request rate limit (per second, per policy).
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure VoIP profiles.
    fortios_voip_profile:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      voip_profile:
        state: "present"
        comment: "Comment."
        name: "default_name_4"
        sccp:
            block-mcast: "disable"
            log-call-summary: "disable"
            log-violations: "disable"
            max-calls: "9"
            status: "disable"
            verify-header: "disable"
        sip:
            ack-rate: "13"
            block-ack: "disable"
            block-bye: "disable"
            block-cancel: "disable"
            block-geo-red-options: "disable"
            block-info: "disable"
            block-invite: "disable"
            block-long-lines: "disable"
            block-message: "disable"
            block-notify: "disable"
            block-options: "disable"
            block-prack: "disable"
            block-publish: "disable"
            block-refer: "disable"
            block-register: "disable"
            block-subscribe: "disable"
            block-unknown: "disable"
            block-update: "disable"
            bye-rate: "31"
            call-keepalive: "32"
            cancel-rate: "33"
            contact-fixup: "disable"
            hnt-restrict-source-ip: "disable"
            hosted-nat-traversal: "disable"
            info-rate: "37"
            invite-rate: "38"
            ips-rtp: "disable"
            log-call-summary: "disable"
            log-violations: "disable"
            malformed-header-allow: "discard"
            malformed-header-call-id: "discard"
            malformed-header-contact: "discard"
            malformed-header-content-length: "discard"
            malformed-header-content-type: "discard"
            malformed-header-cseq: "discard"
            malformed-header-expires: "discard"
            malformed-header-from: "discard"
            malformed-header-max-forwards: "discard"
            malformed-header-p-asserted-identity: "discard"
            malformed-header-rack: "discard"
            malformed-header-record-route: "discard"
            malformed-header-route: "discard"
            malformed-header-rseq: "discard"
            malformed-header-sdp-a: "discard"
            malformed-header-sdp-b: "discard"
            malformed-header-sdp-c: "discard"
            malformed-header-sdp-i: "discard"
            malformed-header-sdp-k: "discard"
            malformed-header-sdp-m: "discard"
            malformed-header-sdp-o: "discard"
            malformed-header-sdp-r: "discard"
            malformed-header-sdp-s: "discard"
            malformed-header-sdp-t: "discard"
            malformed-header-sdp-v: "discard"
            malformed-header-sdp-z: "discard"
            malformed-header-to: "discard"
            malformed-header-via: "discard"
            malformed-request-line: "discard"
            max-body-length: "71"
            max-dialogs: "72"
            max-idle-dialogs: "73"
            max-line-length: "74"
            message-rate: "75"
            nat-trace: "disable"
            no-sdp-fixup: "disable"
            notify-rate: "78"
            open-contact-pinhole: "disable"
            open-record-route-pinhole: "disable"
            open-register-pinhole: "disable"
            open-via-pinhole: "disable"
            options-rate: "83"
            prack-rate: "84"
            preserve-override: "disable"
            provisional-invite-expiry-time: "86"
            publish-rate: "87"
            refer-rate: "88"
            register-contact-trace: "disable"
            register-rate: "90"
            rfc2543-branch: "disable"
            rtp: "disable"
            ssl-algorithm: "high"
            ssl-auth-client: "<your_own_value> (source user.peer.name user.peergrp.name)"
            ssl-auth-server: "<your_own_value> (source user.peer.name user.peergrp.name)"
            ssl-client-certificate: "<your_own_value> (source vpn.certificate.local.name)"
            ssl-client-renegotiation: "allow"
            ssl-max-version: "ssl-3.0"
            ssl-min-version: "ssl-3.0"
            ssl-mode: "off"
            ssl-pfs: "require"
            ssl-send-empty-frags: "enable"
            ssl-server-certificate: "<your_own_value> (source vpn.certificate.local.name)"
            status: "disable"
            strict-register: "disable"
            subscribe-rate: "106"
            unknown-header: "discard"
            update-rate: "108"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule


def login(data, fos):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_voip_profile_data(json):
    option_list = ['comment', 'name', 'sccp',
                   'sip']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def voip_profile(data, fos):
    vdom = data['vdom']
    voip_profile_data = data['voip_profile']
    filtered_data = filter_voip_profile_data(voip_profile_data)

    if voip_profile_data['state'] == "present":
        return fos.set('voip',
                       'profile',
                       data=filtered_data,
                       vdom=vdom)

    elif voip_profile_data['state'] == "absent":
        return fos.delete('voip',
                          'profile',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_voip(data, fos):
    login(data, fos)

    if data['voip_profile']:
        resp = voip_profile(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "voip_profile": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "comment": {"required": False, "type": "str"},
                "name": {"required": True, "type": "str"},
                "sccp": {"required": False, "type": "dict",
                         "options": {
                             "block-mcast": {"required": False, "type": "str",
                                             "choices": ["disable", "enable"]},
                             "log-call-summary": {"required": False, "type": "str",
                                                  "choices": ["disable", "enable"]},
                             "log-violations": {"required": False, "type": "str",
                                                "choices": ["disable", "enable"]},
                             "max-calls": {"required": False, "type": "int"},
                             "status": {"required": False, "type": "str",
                                        "choices": ["disable", "enable"]},
                             "verify-header": {"required": False, "type": "str",
                                               "choices": ["disable", "enable"]}
                         }},
                "sip": {"required": False, "type": "dict",
                        "options": {
                            "ack-rate": {"required": False, "type": "int"},
                            "block-ack": {"required": False, "type": "str",
                                          "choices": ["disable", "enable"]},
                            "block-bye": {"required": False, "type": "str",
                                          "choices": ["disable", "enable"]},
                            "block-cancel": {"required": False, "type": "str",
                                             "choices": ["disable", "enable"]},
                            "block-geo-red-options": {"required": False, "type": "str",
                                                      "choices": ["disable", "enable"]},
                            "block-info": {"required": False, "type": "str",
                                           "choices": ["disable", "enable"]},
                            "block-invite": {"required": False, "type": "str",
                                             "choices": ["disable", "enable"]},
                            "block-long-lines": {"required": False, "type": "str",
                                                 "choices": ["disable", "enable"]},
                            "block-message": {"required": False, "type": "str",
                                              "choices": ["disable", "enable"]},
                            "block-notify": {"required": False, "type": "str",
                                             "choices": ["disable", "enable"]},
                            "block-options": {"required": False, "type": "str",
                                              "choices": ["disable", "enable"]},
                            "block-prack": {"required": False, "type": "str",
                                            "choices": ["disable", "enable"]},
                            "block-publish": {"required": False, "type": "str",
                                              "choices": ["disable", "enable"]},
                            "block-refer": {"required": False, "type": "str",
                                            "choices": ["disable", "enable"]},
                            "block-register": {"required": False, "type": "str",
                                               "choices": ["disable", "enable"]},
                            "block-subscribe": {"required": False, "type": "str",
                                                "choices": ["disable", "enable"]},
                            "block-unknown": {"required": False, "type": "str",
                                              "choices": ["disable", "enable"]},
                            "block-update": {"required": False, "type": "str",
                                             "choices": ["disable", "enable"]},
                            "bye-rate": {"required": False, "type": "int"},
                            "call-keepalive": {"required": False, "type": "int"},
                            "cancel-rate": {"required": False, "type": "int"},
                            "contact-fixup": {"required": False, "type": "str",
                                              "choices": ["disable", "enable"]},
                            "hnt-restrict-source-ip": {"required": False, "type": "str",
                                                       "choices": ["disable", "enable"]},
                            "hosted-nat-traversal": {"required": False, "type": "str",
                                                     "choices": ["disable", "enable"]},
                            "info-rate": {"required": False, "type": "int"},
                            "invite-rate": {"required": False, "type": "int"},
                            "ips-rtp": {"required": False, "type": "str",
                                        "choices": ["disable", "enable"]},
                            "log-call-summary": {"required": False, "type": "str",
                                                 "choices": ["disable", "enable"]},
                            "log-violations": {"required": False, "type": "str",
                                               "choices": ["disable", "enable"]},
                            "malformed-header-allow": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-call-id": {"required": False, "type": "str",
                                                         "choices": ["discard", "pass", "respond"]},
                            "malformed-header-contact": {"required": False, "type": "str",
                                                         "choices": ["discard", "pass", "respond"]},
                            "malformed-header-content-length": {"required": False, "type": "str",
                                                                "choices": ["discard", "pass", "respond"]},
                            "malformed-header-content-type": {"required": False, "type": "str",
                                                              "choices": ["discard", "pass", "respond"]},
                            "malformed-header-cseq": {"required": False, "type": "str",
                                                      "choices": ["discard", "pass", "respond"]},
                            "malformed-header-expires": {"required": False, "type": "str",
                                                         "choices": ["discard", "pass", "respond"]},
                            "malformed-header-from": {"required": False, "type": "str",
                                                      "choices": ["discard", "pass", "respond"]},
                            "malformed-header-max-forwards": {"required": False, "type": "str",
                                                              "choices": ["discard", "pass", "respond"]},
                            "malformed-header-p-asserted-identity": {"required": False, "type": "str",
                                                                     "choices": ["discard", "pass", "respond"]},
                            "malformed-header-rack": {"required": False, "type": "str",
                                                      "choices": ["discard", "pass", "respond"]},
                            "malformed-header-record-route": {"required": False, "type": "str",
                                                              "choices": ["discard", "pass", "respond"]},
                            "malformed-header-route": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-rseq": {"required": False, "type": "str",
                                                      "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-a": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-b": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-c": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-i": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-k": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-m": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-o": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-r": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-s": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-t": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-v": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-sdp-z": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "malformed-header-to": {"required": False, "type": "str",
                                                    "choices": ["discard", "pass", "respond"]},
                            "malformed-header-via": {"required": False, "type": "str",
                                                     "choices": ["discard", "pass", "respond"]},
                            "malformed-request-line": {"required": False, "type": "str",
                                                       "choices": ["discard", "pass", "respond"]},
                            "max-body-length": {"required": False, "type": "int"},
                            "max-dialogs": {"required": False, "type": "int"},
                            "max-idle-dialogs": {"required": False, "type": "int"},
                            "max-line-length": {"required": False, "type": "int"},
                            "message-rate": {"required": False, "type": "int"},
                            "nat-trace": {"required": False, "type": "str",
                                          "choices": ["disable", "enable"]},
                            "no-sdp-fixup": {"required": False, "type": "str",
                                             "choices": ["disable", "enable"]},
                            "notify-rate": {"required": False, "type": "int"},
                            "open-contact-pinhole": {"required": False, "type": "str",
                                                     "choices": ["disable", "enable"]},
                            "open-record-route-pinhole": {"required": False, "type": "str",
                                                          "choices": ["disable", "enable"]},
                            "open-register-pinhole": {"required": False, "type": "str",
                                                      "choices": ["disable", "enable"]},
                            "open-via-pinhole": {"required": False, "type": "str",
                                                 "choices": ["disable", "enable"]},
                            "options-rate": {"required": False, "type": "int"},
                            "prack-rate": {"required": False, "type": "int"},
                            "preserve-override": {"required": False, "type": "str",
                                                  "choices": ["disable", "enable"]},
                            "provisional-invite-expiry-time": {"required": False, "type": "int"},
                            "publish-rate": {"required": False, "type": "int"},
                            "refer-rate": {"required": False, "type": "int"},
                            "register-contact-trace": {"required": False, "type": "str",
                                                       "choices": ["disable", "enable"]},
                            "register-rate": {"required": False, "type": "int"},
                            "rfc2543-branch": {"required": False, "type": "str",
                                               "choices": ["disable", "enable"]},
                            "rtp": {"required": False, "type": "str",
                                    "choices": ["disable", "enable"]},
                            "ssl-algorithm": {"required": False, "type": "str",
                                              "choices": ["high", "medium", "low"]},
                            "ssl-auth-client": {"required": False, "type": "str"},
                            "ssl-auth-server": {"required": False, "type": "str"},
                            "ssl-client-certificate": {"required": False, "type": "str"},
                            "ssl-client-renegotiation": {"required": False, "type": "str",
                                                         "choices": ["allow", "deny", "secure"]},
                            "ssl-max-version": {"required": False, "type": "str",
                                                "choices": ["ssl-3.0", "tls-1.0", "tls-1.1",
                                                            "tls-1.2"]},
                            "ssl-min-version": {"required": False, "type": "str",
                                                "choices": ["ssl-3.0", "tls-1.0", "tls-1.1",
                                                            "tls-1.2"]},
                            "ssl-mode": {"required": False, "type": "str",
                                         "choices": ["off", "full"]},
                            "ssl-pfs": {"required": False, "type": "str",
                                        "choices": ["require", "deny", "allow"]},
                            "ssl-send-empty-frags": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                            "ssl-server-certificate": {"required": False, "type": "str"},
                            "status": {"required": False, "type": "str",
                                       "choices": ["disable", "enable"]},
                            "strict-register": {"required": False, "type": "str",
                                                "choices": ["disable", "enable"]},
                            "subscribe-rate": {"required": False, "type": "int"},
                            "unknown-header": {"required": False, "type": "str",
                                               "choices": ["discard", "pass", "respond"]},
                            "update-rate": {"required": False, "type": "int"}
                        }}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_voip(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Andrew Klychkov (@Andersson007) <aaklychkov@mail.ru>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = r'''
---
module: postgresql_ping
short_description: Check remote PostgreSQL server availability
description:
- Simple module to check remote PostgreSQL server availability.
version_added: '2.8'
options:
  db:
    description:
    - Name of database to connect.
    type: str
    aliases:
    - login_db
notes:
- The default authentication assumes that you are either logging in as or
  sudo'ing to the postgres account on the host.
- This module uses psycopg2, a Python PostgreSQL database adapter. You must
  ensure that psycopg2 is installed on the host before using this module. If
  the remote host is the PostgreSQL server (which is the default case), then
  PostgreSQL must also be installed on the remote host. For Ubuntu-based
  systems, install the postgresql, libpq-dev, and python-psycopg2 packages
  on the remote host before using this module.
requirements: [ psycopg2 ]
author:
- Andrew Klychkov (@Andersson007)
extends_documentation_fragment: postgres
'''

EXAMPLES = r'''
# PostgreSQL ping dbsrv server from the shell:
# ansible dbsrv -m postgresql_ping

# In the example below you need to generate certificates previously.
# See https://www.postgresql.org/docs/current/libpq-ssl.html for more information.
- name: PostgreSQL ping dbsrv server using not default credentials and ssl
  postgresql_ping:
    db: protected_db
    login_host: dbsrv
    login_user: secret
    login_password: secret_pass
    ca_cert: /root/root.crt
    ssl_mode: verify-full
'''

RETURN = r'''
is_available:
  description: PostgreSQL server availability.
  returned: always
  type: bool
  sample: true
server_version:
  description: PostgreSQL server version.
  returned: always
  type: dict
  sample: { major: 10, minor: 1 }
'''

try:
    from psycopg2.extras import DictCursor
except ImportError:
    # psycopg2 is checked by connect_to_db()
    # from ansible.module_utils.postgres
    pass

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.database import SQLParseError
from ansible.module_utils.postgres import connect_to_db, get_conn_params, postgres_common_argument_spec
from ansible.module_utils._text import to_native
from ansible.module_utils.six import iteritems


# ===========================================
# PostgreSQL module specific support methods.
#


class PgPing(object):
    def __init__(self, module, cursor):
        self.module = module
        self.cursor = cursor
        self.is_available = False
        self.version = {}

    def do(self):
        self.get_pg_version()
        return (self.is_available, self.version)

    def get_pg_version(self):
        query = "SELECT version()"
        raw = self.__exec_sql(query)[0][0]
        if raw:
            self.is_available = True
            raw = raw.split()[1].split('.')
            self.version = dict(
                major=int(raw[0]),
                minor=int(raw[1]),
            )

    def __exec_sql(self, query):
        try:
            self.cursor.execute(query)
            res = self.cursor.fetchall()
            if res:
                return res
        except Exception as e:
            self.module.fail_json("Unable to execute '%s': %s" % (query, to_native(e)))

        return False

# ===========================================
# Module execution.
#


def main():
    argument_spec = postgres_common_argument_spec()
    argument_spec.update(
        db=dict(type='str', aliases=['login_db']),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    # Set some default values:
    cursor = False
    db_connection = False
    result = dict(
        changed=False,
        is_available=False,
        server_version=dict(),
    )

    conn_params = get_conn_params(module, module.params)
    db_connection = connect_to_db(module, conn_params, fail_on_conn=False)

    if db_connection is not None:
        cursor = db_connection.cursor(cursor_factory=DictCursor)

    # Do job:
    pg_ping = PgPing(module, cursor)
    if cursor:
        # If connection established:
        result["is_available"], result["server_version"] = pg_ping.do()
        db_connection.rollback()

    module.exit_json(**result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright (c) 2019 Ansible Project
# Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

from ansible.module_utils._text import to_native
from ansible.module_utils.common._collections_compat import Mapping
from ansible.module_utils.common.collections import is_iterable

from ansible.module_utils.six import (
    binary_type,
    integer_types,
    text_type,
)

# Python2 & 3 way to get NoneType
NoneType = type(None)

# if adding boolean attribute, also add to PASS_BOOL
# some of this dupes defaults from controller config
PASS_VARS = {
    'check_mode': ('check_mode', False),
    'debug': ('_debug', False),
    'diff': ('_diff', False),
    'keep_remote_files': ('_keep_remote_files', False),
    'module_name': ('_name', None),
    'no_log': ('no_log', False),
    'remote_tmp': ('_remote_tmp', None),
    'selinux_special_fs': ('_selinux_special_fs', ['fuse', 'nfs', 'vboxsf', 'ramfs', '9p']),
    'shell_executable': ('_shell', '/bin/sh'),
    'socket': ('_socket_path', None),
    'string_conversion_action': ('_string_conversion_action', 'warn'),
    'syslog_facility': ('_syslog_facility', 'INFO'),
    'tmpdir': ('_tmpdir', None),
    'verbosity': ('_verbosity', 0),
    'version': ('ansible_version', '0.0'),
}

PASS_BOOLS = ('check_mode', 'debug', 'diff', 'keep_remote_files', 'no_log')


def _return_datastructure_name(obj):
    """ Return native stringified values from datastructures.

    For use with removing sensitive values pre-jsonification."""
    if isinstance(obj, (text_type, binary_type)):
        if obj:
            yield to_native(obj, errors='surrogate_or_strict')
        return
    elif isinstance(obj, Mapping):
        for element in obj.items():
            for subelement in _return_datastructure_name(element[1]):
                yield subelement
    elif is_iterable(obj):
        for element in obj:
            for subelement in _return_datastructure_name(element):
                yield subelement
    elif isinstance(obj, (bool, NoneType)):
        # This must come before int because bools are also ints
        return
    elif isinstance(obj, tuple(list(integer_types) + [float])):
        yield to_native(obj, nonstring='simplerepr')
    else:
        raise TypeError('Unknown parameter type: %s, %s' % (type(obj), obj))


def list_no_log_values(argument_spec, params):
    """Return set of no log values

    :arg argument_spec: An argument spec dictionary from a module
    :arg params: Dictionary of all module parameters

    :returns: Set of strings that should be hidden from output::

        {'secret_dict_value', 'secret_list_item_one', 'secret_list_item_two', 'secret_string'}
    """

    no_log_values = set()
    for arg_name, arg_opts in argument_spec.items():
        if arg_opts.get('no_log', False):
            # Find the value for the no_log'd param
            no_log_object = params.get(arg_name, None)

            if no_log_object:
                no_log_values.update(_return_datastructure_name(no_log_object))

    return no_log_values


def list_deprecations(argument_spec, params):
    """Return a list of deprecations

    :arg argument_spec: An argument spec dictionary from a module
    :arg params: Dictionary of all module parameters

    :returns: List of dictionaries containing a message and version in which
        the deprecated parameter will be removed, or an empty list::

            [{'msg': "Param 'deptest' is deprecated. See the module docs for more information", 'version': '2.9'}]
    """

    deprecations = []
    for arg_name, arg_opts in argument_spec.items():
        if arg_opts.get('removed_in_version') is not None and arg_name in params:
            deprecations.append({
                'msg': "Param '%s' is deprecated. See the module docs for more information" % arg_name,
                'version': arg_opts.get('removed_in_version')
            })

    return deprecations


def handle_aliases(argument_spec, params):
    """Return a two item tuple. The first is a dictionary of aliases, the second is
    a list of legal inputs."""

    legal_inputs = ['_ansible_%s' % k for k in PASS_VARS]
    aliases_results = {}  # alias:canon

    for (k, v) in argument_spec.items():
        legal_inputs.append(k)
        aliases = v.get('aliases', None)
        default = v.get('default', None)
        required = v.get('required', False)
        if default is not None and required:
            # not alias specific but this is a good place to check this
            raise ValueError("internal error: required and default are mutually exclusive for %s" % k)
        if aliases is None:
            continue
        if not is_iterable(aliases) or isinstance(aliases, (binary_type, text_type)):
            raise TypeError('internal error: aliases must be a list or tuple')
        for alias in aliases:
            legal_inputs.append(alias)
            aliases_results[alias] = k
            if alias in params:
                params[k] = params[alias]

    return aliases_results, legal_inputs



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# (c) 2018, Gaudenz Steinlin <gaudenz.steinlin@cloudscale.ch>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: cloudscale_volume
short_description: Manages volumes on the cloudscale.ch IaaS service
description:
  - Create, attach/detach and delete volumes on the cloudscale.ch IaaS service.
notes:
  - To create a new volume at least the I(name) and I(size_gb) options
    are required.
  - A volume can be created and attached to a server in the same task.
version_added: 2.8
author: "Gaudenz Steinlin (@gaudenz)"
options:
  state:
    description:
      - State of the volume.
    default: present
    choices: [ present, absent ]
  name:
    description:
      - Name of the volume. Either name or UUID must be present to change an
        existing volume.
  uuid:
    description:
      - UUID of the volume. Either name or UUID must be present to change an
        existing volume.
  size_gb:
    description:
      - Size of the volume in GB.
  type:
    description:
      - Type of the volume. Cannot be changed after creating the volume.
        Defaults to ssd on volume creation.
    choices: [ ssd, bulk ]
  server_uuids:
    description:
      - UUIDs of the servers this volume is attached to. Set this to [] to
        detach the volume. Currently a volume can only be attached to a
        single server.
    aliases: [ server_uuid ]
extends_documentation_fragment: cloudscale
'''

EXAMPLES = '''
# Create a new SSD volume
- name: Create an SSD volume
  cloudscale_volume:
    name: my_ssd_volume
    size_gb: 50
    api_token: xxxxxx
  register: my_ssd_volume

# Attach an existing volume to a server
- name: Attach volume to server
  cloudscale_volume:
    uuid: my_ssd_volume.uuid
    server_uuids:
      - ea3b39a3-77a8-4d0b-881d-0bb00a1e7f48
    api_token: xxxxxx

# Create and attach a volume to a server
- name: Create and attach volume to server
  cloudscale_volume:
    name: my_ssd_volume
    size_gb: 50
    server_uuids:
      - ea3b39a3-77a8-4d0b-881d-0bb00a1e7f48
    api_token: xxxxxx

# Detach volume from server
- name: Detach volume from server
  cloudscale_volume:
    uuid: my_ssd_volume.uuid
    server_uuids: []
    api_token: xxxxxx

# Delete a volume
- name: Delete volume
  cloudscale_volume:
    name: my_ssd_volume
    state: absent
    api_token: xxxxxx
'''

RETURN = '''
href:
  description: The API URL to get details about this volume.
  returned: success when state == present
  type: str
  sample: https://api.cloudscale.ch/v1/volumes/2db69ba3-1864-4608-853a-0771b6885a3a
uuid:
  description: The unique identifier for this volume.
  returned: success when state == present
  type: str
  sample: 2db69ba3-1864-4608-853a-0771b6885a3a
name:
  description: The display name of the volume.
  returned: success when state == present
  type: str
  sample: my_ssd_volume
size_gb:
  description: The size of the volume in GB.
  returned: success when state == present
  type: str
  sample: 50
type:
  description: "The type of the volume. There are currently two options:
                ssd (default) or bulk."
  returned: success when state == present
  type: str
  sample: bulk
server_uuids:
  description: The UUIDs of the servers this volume is attached to.
  returned: success when state == present
  type: list
  sample: ['47cec963-fcd2-482f-bdb6-24461b2d47b1']
state:
  description: The current status of the volume.
  returned: success
  type: str
  sample: present
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.cloudscale import (AnsibleCloudscaleBase,
                                             cloudscale_argument_spec,
                                             )


class AnsibleCloudscaleVolume(AnsibleCloudscaleBase):

    def __init__(self, module):
        super(AnsibleCloudscaleVolume, self).__init__(module)
        params = self._module.params
        self.info = {
            'name': params['name'],
            'uuid': params['uuid'],
            'state': 'absent',
        }
        self.changed = False
        if params['uuid'] is not None:
            vol = self._get('volumes/%s' % params['uuid'])
            if vol is None and params['state'] == 'present':
                self._module.fail_json(
                    msg='Volume with UUID %s does not exist. Can\'t create a '
                        'volume with a predefined UUID.' % params['uuid'],
                )
            elif vol is not None:
                self.info = vol
                self.info['state'] = 'present'
        else:
            resp = self._get('volumes')
            volumes = [vol for vol in resp if vol['name'] == params['name']]
            if len(volumes) == 1:
                self.info = volumes[0]
                self.info['state'] = 'present'
            elif len(volumes) > 1:
                self._module.fail_json(
                    msg='More than 1 volume with name "%s" exists.'
                        % params['name'],
                )

    def create(self):
        params = self._module.params

        # check for required parameters to create a volume
        missing_parameters = []
        for p in ('name', 'size_gb'):
            if p not in params or not params[p]:
                missing_parameters.append(p)

        if len(missing_parameters) > 0:
            self._module.fail_json(
                msg='Missing required parameter(s) to create a volume: %s.'
                    % ' '.join(missing_parameters),
            )

        data = {
            'name': params['name'],
            'size_gb': params['size_gb'],
            'type': params['type'] or 'ssd',
            'server_uuids': params['server_uuids'] or [],
        }

        self.info = self._post('volumes', data)
        self.info['state'] = 'present'
        self.changed = True

    def delete(self):
        self._delete('volumes/%s' % self.info['uuid'])
        self.info = {
            'name': self.info['name'],
            'uuid': self.info['uuid'],
            'state': 'absent',
        }
        self.changed = True

    def update(self, param):
        self._patch(
            'volumes/%s' % self.info['uuid'],
            {param: self._module.params[param]},
        )
        self.info[param] = self._module.params[param]
        self.changed = True


def main():
    argument_spec = cloudscale_argument_spec()
    argument_spec.update(dict(
        state=dict(default='present', choices=('present', 'absent')),
        name=dict(),
        uuid=dict(),
        size_gb=dict(type='int'),
        type=dict(choices=('ssd', 'bulk')),
        server_uuids=dict(type='list', aliases=['server_uuid']),
    ))

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=(('name', 'uuid'),),
        mutually_exclusive=(('name', 'uuid'),),
        supports_check_mode=True,
    )

    volume = AnsibleCloudscaleVolume(module)
    if module.check_mode:
        changed = False
        for param, conv in (('state', str),
                            ('server_uuids', set),
                            ('size_gb', int)):
            if module.params[param] is None:
                continue

            if conv(volume.info[param]) != conv(module.params[param]):
                changed = True
                break

        module.exit_json(changed=changed,
                         **volume.info)

    if (volume.info['state'] == 'absent'
       and module.params['state'] == 'present'):
        volume.create()
    elif (volume.info['state'] == 'present'
          and module.params['state'] == 'absent'):
        volume.delete()

    if module.params['state'] == 'present':
        if (module.params['type'] is not None
           and volume.info['type'] != module.params['type']):
            module.fail_json(
                msg='Cannot change type of an existing volume.',
            )

        for param, conv in (('server_uuids', set), ('size_gb', int)):
            if module.params[param] is None:
                continue
            if conv(volume.info[param]) != conv(module.params[param]):
                volume.update(param)

    module.exit_json(changed=volume.changed, **volume.info)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 Huawei
# GNU General Public License v3.0+ (see COPYING or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

###############################################################################
# Documentation
###############################################################################

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ["preview"],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: hwc_network_vpc
description:
    - Represents an vpc resource.
short_description: Creates a Huawei Cloud VPC
version_added: '2.8'
author: Huawei Inc. (@huaweicloud)
requirements:
    - requests >= 2.18.4
    - keystoneauth1 >= 3.6.0
options:
    state:
        description:
            - Whether the given object should exist in vpc.
        type: str
        choices: ['present', 'absent']
        default: 'present'
    name:
        description:
            - the name of vpc.
        type: str
        required: true
    cidr:
        description:
            - the range of available subnets in the vpc.
        type: str
        required: true
extends_documentation_fragment: hwc
'''

EXAMPLES = '''
- name: create a vpc
  hwc_network_vpc:
      identity_endpoint: "{{ identity_endpoint }}"
      user: "{{ user }}"
      password: "{{ password }}"
      domain: "{{ domain }}"
      project: "{{ project }}"
      region: "{{ region }}"
      name: "vpc_1"
      cidr: "192.168.100.0/24"
      state: present
'''

RETURN = '''
    id:
        description:
            - the id of vpc.
        type: str
        returned: success
    name:
        description:
            - the name of vpc.
        type: str
        returned: success
    cidr:
        description:
            - the range of available subnets in the vpc.
        type: str
        returned: success
    status:
        description:
            - the status of vpc.
        type: str
        returned: success
    routes:
        description:
            - the route information.
        type: complex
        returned: success
        contains:
            destination:
                description:
                    - the destination network segment of a route.
                type: str
                returned: success
            next_hop:
                description:
                    - the next hop of a route.
                type: str
                returned: success
    enable_shared_snat:
        description:
            - show whether the shared snat is enabled.
        type: bool
        returned: success
'''

###############################################################################
# Imports
###############################################################################

from ansible.module_utils.hwc_utils import (HwcSession, HwcModule,
                                            DictComparison, navigate_hash,
                                            remove_nones_from_dict,
                                            remove_empty_from_dict,
                                            are_dicts_different)
import json
import re
import time

###############################################################################
# Main
###############################################################################


def main():
    """Main function"""

    module = HwcModule(
        argument_spec=dict(
            state=dict(default='present', choices=['present', 'absent'], type='str'),
            name=dict(required=True, type='str'),
            cidr=dict(required=True, type='str')
        ),
        supports_check_mode=True,
    )
    session = HwcSession(module, 'network')

    state = module.params['state']

    if (not module.params.get("id")) and module.params.get("name"):
        module.params['id'] = get_id_by_name(session)

    fetch = None
    link = self_link(session)
    # the link will include Nones if required format parameters are missed
    if not re.search('/None/|/None$', link):
        fetch = fetch_resource(session, link)
        if fetch:
            fetch = fetch.get('vpc')
    changed = False

    if fetch:
        if state == 'present':
            expect = _get_editable_properties(module)
            current_state = response_to_hash(module, fetch)
            if are_dicts_different(expect, current_state):
                if not module.check_mode:
                    fetch = update(session, self_link(session), [200])
                    fetch = response_to_hash(module, fetch.get('vpc'))
                changed = True
            else:
                fetch = current_state
        else:
            if not module.check_mode:
                delete(session, self_link(session))
                fetch = {}
            changed = True
    else:
        if state == 'present':
            if not module.check_mode:
                fetch = create(session, collection(session), [200])
                fetch = response_to_hash(module, fetch.get('vpc'))
            changed = True
        else:
            fetch = {}

    fetch.update({'changed': changed})

    module.exit_json(**fetch)


def create(session, link, success_codes=None):
    if not success_codes:
        success_codes = [201, 202]
    module = session.module
    r = return_if_object(module, session.post(link, resource_to_create(module)), success_codes)

    wait_done = wait_for_operation(session, 'create', r)

    url = resource_get_url(session, wait_done)
    return fetch_resource(session, url)


def update(session, link, success_codes=None):
    if not success_codes:
        success_codes = [201, 202]
    module = session.module
    r = return_if_object(module, session.put(link, resource_to_update(module)), success_codes)

    wait_done = wait_for_operation(session, 'update', r)

    url = resource_get_url(session, wait_done)
    return fetch_resource(session, url)


def delete(session, link, success_codes=None):
    if not success_codes:
        success_codes = [202, 204]
    return_if_object(session.module, session.delete(link), success_codes, False)

    wait_for_delete(session, link)


def fetch_resource(session, link, success_codes=None):
    if not success_codes:
        success_codes = [200]
    return return_if_object(session.module, session.get(link), success_codes)


def link_wrapper(f):
    def _wrapper(module, *args, **kwargs):
        try:
            return f(module, *args, **kwargs)
        except KeyError as ex:
            module.fail_json(
                msg="Mapping keys(%s) are not found in generating link." % ex)

    return _wrapper


def get_id_by_name(session):
    module = session.module
    name = module.params.get("name")
    link = list_link(session, {'limit': 10, 'marker': '{marker}'})
    not_format_keys = re.findall("={marker}", link)
    none_values = re.findall("=None", link)

    if not (not_format_keys or none_values):
        r = fetch_resource(session, link)
        if r is None:
            return ""
        r = r.get('vpcs', [])
        ids = [
            i.get('id') for i in r if i.get('name', '') == name
        ]
        if not ids:
            return ""
        elif len(ids) == 1:
            return ids[0]
        else:
            module.fail_json(msg="Multiple resources with same name are found.")
    elif none_values:
        module.fail_json(
            msg="Can not find id by name because url includes None.")
    else:
        p = {'marker': ''}
        ids = set()
        while True:
            r = fetch_resource(session, link.format(**p))
            if r is None:
                break
            r = r.get('vpcs', [])
            if r == []:
                break
            for i in r:
                if i.get('name') == name:
                    ids.add(i.get('id'))
            if len(ids) >= 2:
                module.fail_json(
                    msg="Multiple resources with same name are found.")

            p['marker'] = r[-1].get('id')

        return ids.pop() if ids else ""


@link_wrapper
def list_link(session, extra_data=None):
    url = "{endpoint}vpcs?limit={limit}&marker={marker}"

    combined = session.module.params.copy()
    if extra_data:
        combined.update(extra_data)

    combined['endpoint'] = session.get_service_endpoint('vpc')

    return url.format(**combined)


@link_wrapper
def self_link(session):
    url = "{endpoint}vpcs/{id}"

    combined = session.module.params.copy()
    combined['endpoint'] = session.get_service_endpoint('vpc')

    return url.format(**combined)


@link_wrapper
def collection(session):
    url = "{endpoint}vpcs"

    combined = session.module.params.copy()
    combined['endpoint'] = session.get_service_endpoint('vpc')

    return url.format(**combined)


def return_if_object(module, response, success_codes, has_content=True):
    code = response.status_code

    # If not found, return nothing.
    if code == 404:
        return None

    success_codes = [200, 201, 202, 203, 204, 205, 206, 207, 208, 226]
    # If no content, return nothing.
    if code in success_codes and not has_content:
        return None

    result = None
    try:
        result = response.json()
    except getattr(json.decoder, 'JSONDecodeError', ValueError) as inst:
        module.fail_json(msg="Invalid JSON response with error: %s" % inst)

    if code not in success_codes:
        msg = navigate_hash(result, ['message'])
        if msg:
            module.fail_json(msg=msg)
        else:
            module.fail_json(msg="operation failed, return code=%d" % code)

    return result


def resource_to_create(module):
    request = remove_empty_from_dict({
        u'name': module.params.get('name'),
        u'cidr': module.params.get('cidr')
    })
    return {'vpc': request}


def resource_to_update(module):
    request = remove_nones_from_dict({
        u'name': module.params.get('name'),
        u'cidr': module.params.get('cidr')
    })
    return {'vpc': request}


def _get_editable_properties(module):
    request = remove_nones_from_dict({
        "name": module.params.get("name"),
        "cidr": module.params.get("cidr"),
    })

    return request


def response_to_hash(module, response):
    """ Remove unnecessary properties from the response.
        This is for doing comparisons with Ansible's current parameters.
    """
    return {
        u'id': response.get(u'id'),
        u'name': response.get(u'name'),
        u'cidr': response.get(u'cidr'),
        u'status': response.get(u'status'),
        u'routes': VpcRoutesArray(response.get(u'routes', []), module).from_response(),
        u'enable_shared_snat': response.get(u'enable_shared_snat')
    }


@link_wrapper
def resource_get_url(session, wait_done):
    combined = session.module.params.copy()
    combined['op_id'] = navigate_hash(wait_done, ['vpc', 'id'])
    url = 'vpcs/{op_id}'.format(**combined)

    endpoint = session.get_service_endpoint('vpc')
    return endpoint + url


@link_wrapper
def async_op_url(session, extra_data=None):
    url = "{endpoint}vpcs/{op_id}"

    combined = session.module.params.copy()
    if extra_data:
        combined.update(extra_data)

    combined['endpoint'] = session.get_service_endpoint('vpc')

    return url.format(**combined)


def wait_for_operation(session, op_type, op_result):
    op_id = navigate_hash(op_result, ['vpc', 'id'])
    url = async_op_url(session, {'op_id': op_id})
    timeout = 60 * int(session.module.params['timeouts'][op_type].rstrip('m'))
    states = {
        'create': {
            'allowed': ['CREATING', 'DONW', 'OK'],
            'complete': ['OK'],
        },
        'update': {
            'allowed': ['PENDING_UPDATE', 'DONW', 'OK'],
            'complete': ['OK'],
        }
    }

    return wait_for_completion(url, timeout, states[op_type]['allowed'],
                               states[op_type]['complete'], session)


def wait_for_completion(op_uri, timeout, allowed_states,
                        complete_states, session):
    module = session.module
    end = time.time() + timeout
    while time.time() <= end:
        try:
            op_result = fetch_resource(session, op_uri)
        except Exception:
            time.sleep(1.0)
            continue

        raise_if_errors(op_result, module)

        status = navigate_hash(op_result, ['vpc', 'status'])
        if status not in allowed_states:
            module.fail_json(msg="Invalid async operation status %s" % status)
        if status in complete_states:
            return op_result

        time.sleep(1.0)

    module.fail_json(msg="Timeout to wait completion.")


def raise_if_errors(response, module):
    errors = navigate_hash(response, [])
    if errors:
        module.fail_json(msg=navigate_hash(response, []))


def wait_for_delete(session, link):
    end = time.time() + 60 * int(
        session.module.params['timeouts']['delete'].rstrip('m'))
    while time.time() <= end:
        try:
            resp = session.get(link)
            if resp.status_code == 404:
                return
        except Exception:
            pass

        time.sleep(1.0)

    session.module.fail_json(msg="Timeout to wait for deletion to be complete.")


class VpcRoutesArray(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = []

    def to_request(self):
        items = []
        for item in self.request:
            items.append(self._request_for_item(item))
        return items

    def from_response(self):
        items = []
        for item in self.request:
            items.append(self._response_from_item(item))
        return items

    def _request_for_item(self, item):
        return {
            u'destination': item.get('destination'),
            u'nexthop': item.get('next_hop')
        }

    def _response_from_item(self, item):
        return {
            u'destination': item.get(u'destination'),
            u'next_hop': item.get(u'nexthop')
        }


if __name__ == '__main__':
    main()



# Copyright: (c) 2018, Aaron Smith <ajsmith10381@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r"""
lookup: aws_secret
author:
  - Aaron Smith <ajsmith10381@gmail.com>
version_added: "2.8"
requirements:
  - boto3
  - botocore>=1.10.0
extends_documentation_fragment:
  - aws_credentials
short_description: Look up secrets stored in AWS Secrets Manager.
description:
  - Look up secrets stored in AWS Secrets Manager provided the caller
    has the appropriate permissions to read the secret.
  - Lookup is based on the secret's `Name` value.
  - Optional parameters can be passed into this lookup; `version_id` and `version_stage`
options:
  _terms:
    description: Name of the secret to look up in AWS Secrets Manager.
    required: True
  version_id:
    description: Version of the secret(s).
    required: False
  version_stage:
    description: Stage of the secret version.
    required: False
  join:
    description:
        - Join two or more entries to form an extended secret.
        - This is useful for overcoming the 4096 character limit imposed by AWS.
    type: boolean
    default: false
"""

EXAMPLES = r"""
 - name: Create RDS instance with aws_secret lookup for password param
   rds:
     command: create
     instance_name: app-db
     db_engine: MySQL
     size: 10
     instance_type: db.m1.small
     username: dbadmin
     password: "{{ lookup('aws_secret', 'DbSecret') }}"
     tags:
       Environment: staging
"""

RETURN = r"""
_raw:
  description:
    Returns the value of the secret stored in AWS Secrets Manager.
"""

from ansible.errors import AnsibleError

try:
    import boto3
    import botocore
except ImportError:
    raise AnsibleError("The lookup aws_secret requires boto3 and botocore.")

from ansible.plugins import AnsiblePlugin
from ansible.plugins.lookup import LookupBase
from ansible.module_utils._text import to_native


def _boto3_conn(region, credentials):
    boto_profile = credentials.pop('aws_profile', None)

    try:
        connection = boto3.session.Session(profile_name=boto_profile).client('secretsmanager', region, **credentials)
    except (botocore.exceptions.ProfileNotFound, botocore.exceptions.PartialCredentialsError) as e:
        if boto_profile:
            try:
                connection = boto3.session.Session(profile_name=boto_profile).client('secretsmanager', region)
            except (botocore.exceptions.ProfileNotFound, botocore.exceptions.PartialCredentialsError) as e:
                raise AnsibleError("Insufficient credentials found.")
        else:
            raise AnsibleError("Insufficient credentials found.")
    return connection


class LookupModule(LookupBase):
    def _get_credentials(self):
        credentials = {}
        credentials['aws_profile'] = self.get_option('aws_profile')
        credentials['aws_secret_access_key'] = self.get_option('aws_secret_key')
        credentials['aws_access_key_id'] = self.get_option('aws_access_key')
        credentials['aws_session_token'] = self.get_option('aws_security_token')

        # fallback to IAM role credentials
        if not credentials['aws_profile'] and not (credentials['aws_access_key_id'] and credentials['aws_secret_access_key']):
            session = botocore.session.get_session()
            if session.get_credentials() is not None:
                credentials['aws_access_key_id'] = session.get_credentials().access_key
                credentials['aws_secret_access_key'] = session.get_credentials().secret_key
                credentials['aws_session_token'] = session.get_credentials().token

        return credentials

    def run(self, terms, variables, **kwargs):

        self.set_options(var_options=variables, direct=kwargs)
        boto_credentials = self._get_credentials()

        region = self.get_option('region')
        client = _boto3_conn(region, boto_credentials)

        secrets = []
        for term in terms:
            params = {}
            params['SecretId'] = term
            if kwargs.get('version_id'):
                params['VersionId'] = kwargs.get('version_id')
            if kwargs.get('version_stage'):
                params['VersionStage'] = kwargs.get('version_stage')

            try:
                response = client.get_secret_value(**params)
                if 'SecretBinary' in response:
                    secrets.append(response['SecretBinary'])
                if 'SecretString' in response:
                    secrets.append(response['SecretString'])
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                raise AnsibleError("Failed to retrieve secret: %s" % to_native(e))

        if kwargs.get('join'):
            joined_secret = []
            joined_secret.append(''.join(secrets))
            return joined_secret
        else:
            return secrets



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2017, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: bigip_service_policy
short_description: Manages service policies on a BIG-IP.
description:
  - Service policies allow you to configure timers and port misuse rules,
    if enabled, on a per rule or per context basis.
version_added: 2.6
options:
  name:
    description:
      - Name of the service policy.
    required: True
    type: str
  description:
    description:
      - Description of the service policy.
    type: str
  timer_policy:
    description:
      - The timer policy to attach to the service policy.
    type: str
  port_misuse_policy:
    description:
      - The port misuse policy to attach to the service policy.
      - Requires that C(afm) be provisioned to use. If C(afm) is not provisioned, this parameter
        will be ignored.
    type: str
  state:
    description:
      - Whether the resource should exist or not.
    type: str
    choices:
      - present
      - absent
    default: present
  partition:
    description:
      - Device partition to manage resources on.
    type: str
    default: Common
extends_documentation_fragment: f5
author:
  - Tim Rupp (@caphrim007)
  - Wojciech Wypior (@wojtek0806)
'''

EXAMPLES = r'''
- name: Create a service policy
  bigip_service_policy:
    name: foo
    timer_policy: timer1
    port_misuse_policy: misuse1
    timer_policy_enabled: yes
    port_misuse_policy_enabled: yes
    state: present
    provider:
      user: admin
      password: secret
      server: lb.mydomain.com
  delegate_to: localhost
'''

RETURN = r'''
timer_policy:
  description: The new timer policy attached to the resource.
  returned: changed
  type: str
  sample: /Common/timer1
port_misuse_policy:
  description: The new port misuse policy attached to the resource.
  returned: changed
  type: str
  sample: /Common/misuse1
description:
  description: New description of the resource.
  returned: changed
  type: str
  sample: My service policy description
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.basic import env_fallback

try:
    from library.module_utils.network.f5.bigip import F5RestClient
    from library.module_utils.network.f5.common import F5ModuleError
    from library.module_utils.network.f5.common import AnsibleF5Parameters
    from library.module_utils.network.f5.common import f5_argument_spec
    from library.module_utils.network.f5.common import fq_name
    from library.module_utils.network.f5.common import transform_name
    from library.module_utils.network.f5.icontrol import module_provisioned
except ImportError:
    from ansible.module_utils.network.f5.bigip import F5RestClient
    from ansible.module_utils.network.f5.common import F5ModuleError
    from ansible.module_utils.network.f5.common import AnsibleF5Parameters
    from ansible.module_utils.network.f5.common import f5_argument_spec
    from ansible.module_utils.network.f5.common import fq_name
    from ansible.module_utils.network.f5.common import transform_name
    from ansible.module_utils.network.f5.icontrol import module_provisioned


class Parameters(AnsibleF5Parameters):
    api_map = {
        'portMisusePolicy': 'port_misuse_policy',
        'timerPolicy': 'timer_policy',
    }

    api_attributes = [
        'description',
        'timerPolicy',
        'portMisusePolicy',
    ]

    returnables = [
        'description',
        'timer_policy',
        'port_misuse_policy',
    ]

    updatables = [
        'description',
        'timer_policy',
        'port_misuse_policy',
    ]


class ApiParameters(Parameters):
    pass


class ModuleParameters(Parameters):
    @property
    def timer_policy(self):
        if self._values['timer_policy'] is None:
            return None
        if self._values['timer_policy'] == '':
            return ''
        return fq_name(self.partition, self._values['timer_policy'])

    @property
    def port_misuse_policy(self):
        if self._values['port_misuse_policy'] is None:
            return None
        if self._values['port_misuse_policy'] == '':
            return ''
        return fq_name(self.partition, self._values['port_misuse_policy'])


class Changes(Parameters):
    def to_return(self):
        result = {}
        try:
            for returnable in self.returnables:
                result[returnable] = getattr(self, returnable)
            result = self._filter_params(result)
        except Exception:
            pass
        return result


class UsableChanges(Changes):
    pass


class ReportableChanges(Changes):
    pass


class Difference(object):
    def __init__(self, want, have=None):
        self.want = want
        self.have = have

    def compare(self, param):
        try:
            result = getattr(self, param)
            return result
        except AttributeError:
            return self.__default(param)

    def __default(self, param):
        attr1 = getattr(self.want, param)
        try:
            attr2 = getattr(self.have, param)
            if attr1 != attr2:
                return attr1
        except AttributeError:
            return attr1


class ModuleManager(object):
    def __init__(self, *args, **kwargs):
        self.module = kwargs.get('module', None)
        self.client = F5RestClient(**self.module.params)
        self.want = ModuleParameters(params=self.module.params)
        self.have = ApiParameters()
        self.changes = UsableChanges()

    def _set_changed_options(self):
        changed = {}
        for key in Parameters.returnables:
            if getattr(self.want, key) is not None:
                changed[key] = getattr(self.want, key)
        if changed:
            self.changes = UsableChanges(params=changed)

    def _update_changed_options(self):
        diff = Difference(self.want, self.have)
        updatables = Parameters.updatables
        changed = dict()
        for k in updatables:
            change = diff.compare(k)
            if change is None:
                continue
            else:
                if isinstance(change, dict):
                    changed.update(change)
                else:
                    changed[k] = change
        if changed:
            self.changes = UsableChanges(params=changed)
            return True
        return False

    def should_update(self):
        result = self._update_changed_options()
        if result:
            return True
        return False

    def exec_module(self):
        changed = False
        result = dict()
        state = self.want.state

        if state == "present":
            changed = self.present()
        elif state == "absent":
            changed = self.absent()

        reportable = ReportableChanges(params=self.changes.to_return())
        changes = reportable.to_return()
        result.update(**changes)
        result.update(dict(changed=changed))
        self._announce_deprecations(result)
        return result

    def _announce_deprecations(self, result):
        warnings = result.pop('__warnings', [])
        for warning in warnings:
            self.client.module.deprecate(
                msg=warning['msg'],
                version=warning['version']
            )

    def present(self):
        if self.exists():
            return self.update()
        else:
            return self.create()

    def absent(self):
        if self.exists():
            return self.remove()
        return False

    def update(self):
        self.have = self.read_current_from_device()
        if not self.should_update():
            return False
        if self.want.port_misuse_policy:
            if not module_provisioned(self.client, 'afm'):
                raise F5ModuleError(
                    "To configure a 'port_misuse_policy', you must have AFM provisioned."
                )
        if self.module.check_mode:
            return True
        self.update_on_device()
        return True

    def remove(self):
        if self.module.check_mode:
            return True
        self.remove_from_device()
        if self.exists():
            raise F5ModuleError("Failed to delete the resource.")
        return True

    def create(self):
        self._set_changed_options()
        if self.want.port_misuse_policy:
            if not module_provisioned(self.client, 'afm'):
                raise F5ModuleError(
                    "To configure a 'port_misuse_policy', you must have AFM provisioned."
                )
        if self.module.check_mode:
            return True
        self.create_on_device()
        return True

    def exists(self):
        uri = "https://{0}:{1}/mgmt/tm/net/service-policy/{2}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            transform_name(self.want.partition, self.want.name)
        )
        resp = self.client.api.get(uri)
        try:
            response = resp.json()
        except ValueError:
            return False
        if resp.status == 404 or 'code' in response and response['code'] == 404:
            return False
        return True

    def create_on_device(self):
        params = self.changes.api_params()
        params['name'] = self.want.name
        params['partition'] = self.want.partition
        uri = "https://{0}:{1}/mgmt/tm/net/service-policy/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        resp = self.client.api.post(uri, json=params)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] in [400, 403]:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

    def update_on_device(self):
        params = self.changes.api_params()
        uri = "https://{0}:{1}/mgmt/tm/net/service-policy/{2}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            transform_name(self.want.partition, self.want.name)
        )
        resp = self.client.api.patch(uri, json=params)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

    def remove_from_device(self):
        uri = "https://{0}:{1}/mgmt/tm/net/service-policy/{2}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            transform_name(self.want.partition, self.want.name)
        )
        response = self.client.api.delete(uri)
        if response.status == 200:
            return True
        raise F5ModuleError(response.content)

    def read_current_from_device(self):
        uri = "https://{0}:{1}/mgmt/tm/net/service-policy/{2}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            transform_name(self.want.partition, self.want.name)
        )
        resp = self.client.api.get(uri)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        return ApiParameters(params=response)


class ArgumentSpec(object):
    def __init__(self):
        self.supports_check_mode = True
        argument_spec = dict(
            name=dict(),
            description=dict(),
            timer_policy=dict(),
            port_misuse_policy=dict(),
            state=dict(
                default='present',
                choices=['absent', 'present']
            ),
            partition=dict(
                default='Common',
                fallback=(env_fallback, ['F5_PARTITION'])
            )
        )
        self.argument_spec = {}
        self.argument_spec.update(f5_argument_spec)
        self.argument_spec.update(argument_spec)


def main():
    spec = ArgumentSpec()

    module = AnsibleModule(
        argument_spec=spec.argument_spec,
        supports_check_mode=spec.supports_check_mode
    )

    try:
        mm = ModuleManager(module=module)
        results = mm.exec_module()
        module.exit_json(**results)
    except F5ModuleError as ex:
        module.fail_json(msg=str(ex))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2018, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: bigip_apm_policy_fetch
short_description: Exports the APM policy or APM access profile from remote nodes.
description:
  - Exports the apm policy or APM access profile from remote nodes.
version_added: 2.8
options:
  name:
    description:
      - The name of the APM policy or APM access profile exported to create a file on the remote device for downloading.
    type: str
    required: True
  dest:
    description:
      - A directory to save the file into.
    type: path
  file:
    description:
      - The name of the file to be created on the remote device for downloading.
    type: str
  type:
    description:
      - Specifies the type of item to export from device.
    type: str
    choices:
      - profile_access
      - access_policy
    default: profile_access
  force:
    description:
      - If C(no), the file will only be transferred if it does not exist in the the destination.
    type: bool
    default: yes
  partition:
    description:
      - Device partition to which contain APM policy or APM access profile to export.
    type: str
    default: Common
notes:
  - Due to ID685681 it is not possible to execute ng_* tools via REST api on v12.x and 13.x, once this is fixed
    this restriction will be removed.
  - Requires BIG-IP >= 14.0.0
extends_documentation_fragment: f5
author:
  - Wojciech Wypior (@wojtek0806)
'''

EXAMPLES = r'''
- name: Export APM access profile
  bigip_apm_policy_fetch:
    name: foobar
    file: export_foo
    dest: /root/download
    provider:
      password: secret
      server: lb.mydomain.com
      user: admin
  delegate_to: localhost

- name: Export APM access policy
  bigip_apm_policy_fetch:
    name: foobar
    file: export_foo
    dest: /root/download
    type: access_policy
    provider:
      password: secret
      server: lb.mydomain.com
      user: admin
  delegate_to: localhost

- name: Export APM access profile, autogenerate name
  bigip_apm_policy_fetch:
    name: foobar
    dest: /root/download
    provider:
      password: secret
      server: lb.mydomain.com
      user: admin
  delegate_to: localhost
'''

RETURN = r'''
name:
  description: Name of the APM policy or APM access profile to be exported.
  returned: changed
  type: str
  sample: APM_policy_global
file:
  description:
    - Name of the exported file on the remote BIG-IP to download. If not
      specified, then this will be a randomly generated filename.
  returned: changed
  type: str
  sample: foobar_file
dest:
  description: Local path to download exported APM policy.
  returned: changed
  type: str
  sample: /root/downloads/profile-foobar_file.conf.tar.gz
type:
  description: Set to specify type of item to export.
  returned: changed
  type: str
  sample: access_policy
'''

import os
import tempfile

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.basic import env_fallback
from distutils.version import LooseVersion

try:
    from library.module_utils.network.f5.bigip import F5RestClient
    from library.module_utils.network.f5.common import F5ModuleError
    from library.module_utils.network.f5.common import AnsibleF5Parameters
    from library.module_utils.network.f5.common import f5_argument_spec
    from library.module_utils.network.f5.common import transform_name
    from library.module_utils.network.f5.icontrol import download_file
    from library.module_utils.network.f5.icontrol import tmos_version
    from library.module_utils.network.f5.icontrol import module_provisioned
except ImportError:
    from ansible.module_utils.network.f5.bigip import F5RestClient
    from ansible.module_utils.network.f5.common import F5ModuleError
    from ansible.module_utils.network.f5.common import AnsibleF5Parameters
    from ansible.module_utils.network.f5.common import f5_argument_spec
    from ansible.module_utils.network.f5.common import transform_name
    from ansible.module_utils.network.f5.icontrol import download_file
    from ansible.module_utils.network.f5.icontrol import tmos_version
    from ansible.module_utils.network.f5.icontrol import module_provisioned


class Parameters(AnsibleF5Parameters):
    api_map = {}

    api_attributes = []

    returnables = [
        'name',
        'file',
        'dest',
        'type',
        'force',
    ]

    updatables = []


class ApiParameters(Parameters):
    pass


class ModuleParameters(Parameters):
    def _item_exists(self):
        if self.type == 'access_policy':
            uri = 'https://{0}:{1}/mgmt/tm/apm/policy/access-policy/{2}'.format(
                self.client.provider['server'],
                self.client.provider['server_port'],
                transform_name(self.partition, self.name)
            )
        else:
            uri = 'https://{0}:{1}/mgmt/tm/apm/profile/access/{2}'.format(
                self.client.provider['server'],
                self.client.provider['server_port'],
                transform_name(self.partition, self.name)
            )
        resp = self.client.api.get(uri)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))
        if 'items' in response and response['items'] != []:
            return True
        return False

    @property
    def file(self):
        if self._values['file'] is not None:
            return self._values['file']
        result = next(tempfile._get_candidate_names()) + '.tar.gz'
        self._values['file'] = result
        return result

    @property
    def fulldest(self):
        result = None
        if os.path.isdir(self.dest):
            result = os.path.join(self.dest, self.file)
        else:
            if os.path.exists(os.path.dirname(self.dest)):
                result = self.dest
            else:
                try:
                    # os.path.exists() can return false in some
                    # circumstances where the directory does not have
                    # the execute bit for the current user set, in
                    # which case the stat() call will raise an OSError
                    os.stat(os.path.dirname(result))
                except OSError as e:
                    if "permission denied" in str(e).lower():
                        raise F5ModuleError(
                            "Destination directory {0} is not accessible".format(os.path.dirname(result))
                        )
                    raise F5ModuleError(
                        "Destination directory {0} does not exist".format(os.path.dirname(result))
                    )

        if not os.access(os.path.dirname(result), os.W_OK):
            raise F5ModuleError(
                "Destination {0} not writable".format(os.path.dirname(result))
            )
        return result

    @property
    def name(self):
        if not self._item_exists():
            raise F5ModuleError('The provided {0} with the name {1} does not exist on device.'.format(
                self.type, self._values['name'])
            )
        return self._values['name']


class Changes(Parameters):
    def to_return(self):
        result = {}
        try:
            for returnable in self.returnables:
                result[returnable] = getattr(self, returnable)
            result = self._filter_params(result)
        except Exception:
            pass
        return result


class UsableChanges(Changes):
    pass


class ReportableChanges(Changes):
    pass


class ModuleManager(object):
    def __init__(self, *args, **kwargs):
        self.module = kwargs.get('module', None)
        self.client = F5RestClient(**self.module.params)
        self.want = ModuleParameters(params=self.module.params)
        self.changes = UsableChanges()

    def _set_changed_options(self):
        changed = {}
        for key in Parameters.returnables:
            if getattr(self.want, key) is not None:
                changed[key] = getattr(self.want, key)
        if changed:
            self.changes = UsableChanges(params=changed)

    def _announce_deprecations(self, result):
        warnings = result.pop('__warnings', [])
        for warning in warnings:
            self.client.module.deprecate(
                msg=warning['msg'],
                version=warning['version']
            )

    def exec_module(self):
        if not module_provisioned(self.client, 'apm'):
            raise F5ModuleError(
                "APM must be provisioned to use this module."
            )

        if self.version_less_than_14():
            raise F5ModuleError('Due to bug ID685681 it is not possible to use this module on TMOS version below 14.x')

        result = dict()

        self.export()

        reportable = ReportableChanges(params=self.changes.to_return())
        changes = reportable.to_return()
        result.update(**changes)
        result.update(dict(changed=True))
        return result

    def version_less_than_14(self):
        version = tmos_version(self.client)
        if LooseVersion(version) < LooseVersion('14.0.0'):
            return True
        return False

    def export(self):
        if self.exists():
            return self.update()
        else:
            return self.create()

    def update(self):
        if not self.want.force:
            raise F5ModuleError(
                "File '{0}' already exists.".format(self.want.fulldest)
            )
        self.execute()

    def create(self):
        self._set_changed_options()
        if self.module.check_mode:
            return True
        self.create_on_device()
        self.execute()
        return True

    def download(self):
        self.download_from_device(self.want.fulldest)
        if os.path.exists(self.want.fulldest):
            return True
        raise F5ModuleError(
            "Failed to download the remote file."
        )

    def execute(self):
        self.download()
        self.remove_temp_file_from_device()
        return True

    def exists(self):
        if os.path.exists(self.want.fulldest):
            return True
        return False

    def create_on_device(self):
        cmd = 'ng_export -t {0} {1} {2} -p {3}'.format(
            self.want.type, self.want.name, self.want.name, self.want.partition
        )
        uri = "https://{0}:{1}/mgmt/tm/util/bash/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        args = dict(
            command='run',
            utilCmdArgs='-c "{0}"'.format(cmd)
        )
        resp = self.client.api.post(uri, json=args)

        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] in [400, 403]:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        if 'commandResult' in response:
            raise F5ModuleError('Item export command failed.')
        return True

    def _move_file_to_download(self):
        if self.want.type == 'access_policy':
            item = 'policy'
        else:
            item = 'profile'

        name = '{0}-{1}.conf.tar.gz'.format(item, self.want.name)
        move_path = '/shared/tmp/{0} {1}/{2}'.format(
            name,
            '/ts/var/rest',
            self.want.file
        )
        params = dict(
            command='run',
            utilCmdArgs=move_path
        )

        uri = "https://{0}:{1}/mgmt/tm/util/unix-mv/".format(
            self.client.provider['server'],
            self.client.provider['server_port']
        )

        resp = self.client.api.post(uri, json=params)

        try:
            response = resp.json()
            if 'commandResult' in response:
                if 'cannot stat' in response['commandResult']:
                    raise F5ModuleError(response['commandResult'])
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] in [400, 403]:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

        return True

    def download_from_device(self, dest):
        url = 'https://{0}:{1}/mgmt/tm/asm/file-transfer/downloads/{2}'.format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            self.want.file
        )
        try:
            download_file(self.client, url, dest)
        except F5ModuleError:
            raise F5ModuleError(
                "Failed to download the file."
            )
        if os.path.exists(self.want.dest):
            return True
        return False

    def remove_temp_file_from_device(self):
        tpath_name = '/ts/var/rest/{0}'.format(self.want.file)
        uri = "https://{0}:{1}/mgmt/tm/util/unix-rm/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        args = dict(
            command='run',
            utilCmdArgs=tpath_name
        )
        resp = self.client.api.post(uri, json=args)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))
        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)


class ArgumentSpec(object):
    def __init__(self):
        self.supports_check_mode = True
        argument_spec = dict(
            name=dict(
                required=True,
            ),
            dest=dict(
                type='path'
            ),
            type=dict(
                default='profile_access',
                choices=['profile_access', 'access_policy']
            ),
            file=dict(),
            force=dict(
                default='yes',
                type='bool'
            ),
            partition=dict(
                default='Common',
                fallback=(env_fallback, ['F5_PARTITION'])
            )
        )
        self.argument_spec = {}
        self.argument_spec.update(f5_argument_spec)
        self.argument_spec.update(argument_spec)


def main():
    spec = ArgumentSpec()

    module = AnsibleModule(
        argument_spec=spec.argument_spec,
        supports_check_mode=spec.supports_check_mode,
    )

    try:
        mm = ModuleManager(module=module)
        results = mm.exec_module()
        module.exit_json(**results)
    except F5ModuleError as ex:
        module.fail_json(msg=str(ex))


if __name__ == '__main__':
    main()
